import requests
import time
import titlecase

artist = ""
title = ""

def currently_playing(ctime, stop_event):
    """
    Function to get the currently playing track (every 10 seconds).
    """
    while not stop_event.is_set():
        response = requests.get("https://www.grrif.ch/live/covers.json")

        # Check if the request was successful
        if response.status_code == 200:
            ltime = response.json()[3].get("Hours")
            if ctime == 0 or ctime != ltime:
                data = response.json()[3]
                title = titlecase.titlecase(data.get("Title"))
                artist = titlecase.titlecase(data.get("Artist"))
                ctime = ltime
                print(f"Currently playing {title} by {artist}.")
            else:
                pass
        else:
            print("Failed to retrieve data from the URL.")

        time.sleep(10)

#currently_playing(ctime)


"""
GRRIF Archiver helper functions
"""

## [ IMPORTS ]
from datetime import timedelta
import os
import time
import sqlite3
import requests
import titlecase
from bs4 import BeautifulSoup


def plays_to_db(BASE_URL, START_DATE, END_DATE):
    """
    Scrapes the BASE_URL and adds the plays between START_DATE
    and END_DATE to a SQLite database
    """
    database_path = database_handler()
    write_to_db(BASE_URL, START_DATE, END_DATE, database_path)


def database_handler():
    """
    Function to check if the  SQLite database
    with a "plays" table containing the plays exists,
    create it if it doesn't...
    """
    database_path = os.path.join(os.path.expanduser("~"), "grrif_data", "grrif_data.db")

    # Creates the ../data/ directory if it does not exist yet
    if not os.path.isdir(os.path.dirname(database_path)):
        os.makedirs(os.path.dirname(database_path))

    # Create an emtpy db if it does not exist yet
    if not os.path.isfile(database_path):
        # Create the 'plays' table
        conn = sqlite3.connect(database_path)
        conn.execute(
            """CREATE TABLE plays (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            date DATE NOT NULL,
            time TIME NOT NULL,
            artist TEXT NOT NULL,
            title TEXT NOT NULL,
            CONSTRAINT unique_play UNIQUE (date, time)
            );
        """
        )

        conn.commit()
        conn.close()
    else:
        pass

    print(f"Database loaded at {database_path}.")
    return database_path


def write_to_db(base_url, start_date, end_date, database_path):
    """
    Function to scrape the website between start_date and end_date
    and write all the plays to a database
    """
    # Connect to the DB
    conn = sqlite3.connect(database_path)
    c = conn.cursor()

    # Main loop
    current_date = start_date
    while current_date <= end_date:
        # Construct the URL for the current date
        URL = base_url.format(current_date.strftime("%Y-%m-%d"))

        # Send a request to the server and get the response
        response = requests.get(URL)

        # Parse the HTML content of the response using BeautifulSoup
        soup = BeautifulSoup(response.content, "html.parser")

        # Find the section of the page containing the data
        data_section = soup.find("div", {"class": "listing-search-titres"})

        # Find all the data items
        data_items = data_section.find_all("article")

        # Extract the data from each item
        for item in data_items:
            playtime = item.find("div", {"class": "hours"}).text.strip()
            artist = item.find("div", {"class": "artist"}).text.strip()
            title = item.find("div", {"class": "title"}).text.strip()

            # Prettify the data
            pretty_artist = titlecase.titlecase(artist)
            pretty_title = titlecase.titlecase(title)

            # Save into the database
            # Make sure that the entries are not already present
            try:
                c.execute(
                    "INSERT INTO plays (date, time, artist, title) VALUES (?, ?, ?, ?)",
                    (
                        current_date.strftime("%Y-%m-%d"),
                        playtime,
                        pretty_artist,
                        pretty_title,
                    ),
                )
                conn.commit()
            except sqlite3.IntegrityError:
                # print("Error: row already exists")
                continue

        # Update the user on current progress
        print(f"Plays for {current_date.strftime('%Y-%m-%d')} saved to database.")

        # Wait 2 seconds before moving on
        time.sleep(2)

        # Move to the next day
        current_date += timedelta(days=1)

    # When all is over, close the connection to the DB
    conn.close()

    # Print a success message
    print(f"Data archived to {database_path} successfully !")


def plays_to_txt(base_url, start_date, end_date):
    """
    Function to scrape the website between start_date and end_date
    and write all the plays to text files in a YYYY/MM/DD.txt structure
    """
    # Set the path for the files and creates the ../data/plays directory if it does not exist yet
    plays_path = os.path.join(os.path.expanduser("~"), "grrif_data", "plays")
    os.makedirs(
        os.path.join(os.path.expanduser("~"), "grrif_data", "plays"), exist_ok=True
    )

    current_date = start_date
    while current_date <= end_date:
        # Construct the URL for the current date
        URL = base_url.format(current_date.strftime("%Y-%m-%d"))

        # Send a request to the server and get the response
        response = requests.get(URL)

        # Parse the HTML content of the response using BeautifulSoup
        soup = BeautifulSoup(response.content, "html.parser")

        # Find the section of the page containing the data
        data_section = soup.find("div", {"class": "listing-search-titres"})

        # Find all the data items
        data_items = data_section.find_all("article")

        # Extract the data from each item
        for item in reversed(data_items):
            playtime = item.find("div", {"class": "hours"}).text.strip()
            artist = item.find("div", {"class": "artist"}).text.strip()
            title = item.find("div", {"class": "title"}).text.strip()

            # Prettify the data
            pretty_artist = titlecase.titlecase(artist)
            pretty_title = titlecase.titlecase(title)

            # Creates the a /YYYY/MM/ structure as needed within the ../data/plays directory
            dirtree = os.path.join(
                plays_path, current_date.strftime("%Y"), current_date.strftime("%m")
            )
            os.makedirs(dirtree, exist_ok=True)

            # Format and write the data to a DD.txt file
            formatteddata = f"{pretty_artist} - {pretty_title} (@{playtime})"
            currentfile = os.path.join(dirtree, f'{current_date.strftime("%d")}.txt')
            with open(currentfile, "a") as f:
                f.write(formatteddata)
                f.write("\n")

        # Update the user on current progress
        print(f"Plays for {current_date.strftime('%Y-%m-%d')} saved to {currentfile}.")

        # Wait 2 seconds before moving on
        time.sleep(2)

        # Move to the next day
        current_date += timedelta(days=1)

    # Print a success message
    print(f"Data archived to {plays_path} successfully !")


def plays_to_stdout(base_url, start_date, end_date):
    """
    Function to scrape the website between start_date and end_date
    and output the data to stdout
    """
    current_date = start_date
    while current_date <= end_date:
        # Construct the URL for the current date
        URL = base_url.format(current_date.strftime("%Y-%m-%d"))

        # Send a request to the server and get the response
        response = requests.get(URL)

        # Parse the HTML content of the response using BeautifulSoup
        soup = BeautifulSoup(response.content, "html.parser")

        # Find the section of the page containing the data
        data_section = soup.find("div", {"class": "listing-search-titres"})

        # Find all the data items
        data_items = data_section.find_all("article")

        # Extract the data from each item
        for item in reversed(data_items):
            playtime = item.find("div", {"class": "hours"}).text.strip()
            artist = item.find("div", {"class": "artist"}).text.strip()
            title = item.find("div", {"class": "title"}).text.strip()

            # Prettify the data
            pretty_artist = titlecase.titlecase(artist)
            pretty_title = titlecase.titlecase(title)

            # Print the data to stdout
            print(
                f"{pretty_artist} - {pretty_title} (@{playtime} on {current_date.strftime('%Y-%m-%d')})"
            )

        # Wait 2 seconds before moving on
        time.sleep(2)

        # Move to the next day
        current_date += timedelta(days=1)



"""
GRRIF Stats helper functions
"""

import os
import sqlite3
from .grrif_archiver import database_handler

# connect to the SQLite database
database_path = database_handler()
conn = sqlite3.connect(database_path)


def topofthepop(category, ntop, start_date, end_date):
    # Let's cook up a nice query
    query = f"""
    SELECT {category}, COUNT(*) as plays
    FROM plays
    WHERE date BETWEEN ? AND ?
    GROUP BY {category}
    ORDER BY plays DESC
    LIMIT {ntop}
    """
    params = (start_date, end_date)
    results = conn.execute(query, params)

    # print the results for artists
    if category == "artist":
        for row in results:
            print(f"{row[0]} ({row[1]})")
    elif category == "artist, title":
        for row in results:
            print(f"{row[0]} - {row[1]} ({row[2]})")
    # close the database connection
    conn.close()



"""
GRRIF Player helper functions
"""
import os
import requests
import miniaudio
import threading
import time
from .grrif_scrobbler import currently_playing

buffer_file = "buferr.mp3"
max_file_size = 0.25 * 1024 * 1024  # 250 KB
min_play_size = 2 * 1000  # 2 seconds

def stream_and_write(url, file_path, max_size, stop_event):
    response = requests.get(url, stream=True)

    with open(file_path, 'wb') as file:
        buffer = bytearray()
        current_size = 0

        for chunk in response.iter_content(chunk_size=1024):
            if stop_event.is_set():
                break

            buffer.extend(chunk)
            current_size += len(chunk)

            while current_size > max_size:
                buffer.pop(0)
                current_size -= 1024

            file.write(chunk)
            file.flush()

def play_stream(file_path, stop_event):
    print("Buffering...")
    time.sleep(3) # Let the buffer fill up for 3 seconds before playing...

    try:
        stream = miniaudio.stream_file(file_path)

        with miniaudio.PlaybackDevice() as device:
            device.start(stream)
            currently_playing_thread = threading.Thread(target=currently_playing, args=("0", stop_event))
            currently_playing_thread.start()
            input("Streaming. Press the Enter key to stop playback.\n")
            print("Stopping playback, this can take up to 10 seconds.")
    except miniaudio.DecodeError as e:
        print(f"Failed to start streaming: {e}")

    stop_event.set()

def start_playback(quality="mp3_high"):
    stop_event = threading.Event()

    if quality == "mp3_high":
        url = "https://grrif.ice.infomaniak.ch/grrif-high.mp3"
    elif quality == "mp3_low":
        url = "https://grrif.ice.infomaniak.ch/grrif-48.mp3"
    elif quality == "aac_high":
        url = "https://grrif.ice.infomaniak.ch/grrif-128.aac"
    
    if not os.path.exists(buffer_file):
        open(buffer_file, 'w').close()  # Create an empty file if it doesn't exist
        time.sleep(1) # Give the OS a little time to create the file before writing to it

    stream_thread = threading.Thread(target=stream_and_write, args=(url, buffer_file, max_file_size, stop_event))
    stream_thread.start()
    
    play_thread = threading.Thread(target=play_stream, args=(buffer_file, stop_event))
    play_thread.start()

    # Wait for both threads to complete
    stream_thread.join()
    play_thread.join()

    # Delete the buffer file
    if os.path.exists(buffer_file):
        os.remove(buffer_file)


import os
from setuptools import setup, find_packages

with open(
    os.path.join(os.path.abspath(os.path.dirname(__file__)), "README.md"),
    encoding="utf-8",
) as f:
    long_description = f.read()

setup(
    name="grrif_tools",
    description="An unofficial set of tools for Cool Cats™.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Julien 'fetzu' Bono",
    url="https://github.com/fetzu/grrif-tools",
    version="0.7.1",
    download_url="https://github.com/fetzu/grrif-tools/releases/latest",
    packages=find_packages(include=["grrif_tools", "grrif_tools.*"]),
    license="License :: OSI Approved :: MIT License",
    install_requires=[
        "Requests==2.31.0",
        "beautifulsoup4==4.12.2",
        "titlecase==2.4",
        "miniaudio==1.57",
    ],
    entry_points={"console_scripts": ["grrif_tools=grrif_tools.cli:main"]},
)
